@@ -1,34 +1,14 @@
-# --8<--8<--8<--8<--
-#
-# Copyright (C) 2015 Smithsonian Astrophysical Observatory
-#
-# This file is part of MooX::TaggedAttributes
-#
-# MooX::TaggedAttributes is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or (at
-# your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#
-# -->8-->8-->8-->8--
-
 package MooX::TaggedAttributes;
 
+# ABSTRACT: Add a tag with an arbitrary value to a an attribute
+
 use strict;
 use warnings;
 
-our $VERSION = '0.01';
+our $VERSION = '0.07';
 
 use Carp;
-
-use Moo::Role;
+use MRO::Compat;
 
 use Scalar::Util qw[ blessed ];
 use Class::Method::Modifiers qw[ install_modifier ];
@@ -68,48 +48,56 @@ sub import {
     _install_role_import( $target );
 }
 
-sub _install_role_import {
+# this needs to be accessible by tag role import() methods, but don't want it
+# to pollute the namespace
+our $_role_import = sub {
+    my $class = shift;
+    return unless Moo::Role->is_role( $class );
 
+    my $target = caller;
+    Moo::Role->apply_roles_to_package( $target, $class );
+    _install_tags( $target, $TAGSTORE{$class} );
+};
+
+
+sub _install_role_import {
     my $target = shift;
 
-    ## no critic (ProhibitNoStrict)
-    no strict 'refs';
-    no warnings 'redefine';
-    *{"${target}::import"} = sub {
-
-        my $class  = shift;
-        my $target = caller;
-
-        Moo::Role->apply_roles_to_package( $target, $class );
-
-        _install_tags( $target, $TAGSTORE{$class} );
-    };
+    ## no critic (ProhibitStringyEval)
 
+    croak( "error installing import routine into $target\n" )
+      unless eval
+      "package $target; sub import { goto \$MooX::TaggedAttributes::_role_import }; 1;";
 }
 
 
 sub _install_tags {
-
     my ( $target, $tags ) = @_;
 
     if ( $TAGSTORE{$target} ) {
-
         push @{ $TAGSTORE{$target} }, @$tags;
-
     }
 
     else {
-
         $TAGSTORE{$target} = [@$tags];
         _install_tag_handler( $target );
     }
-
 }
 
 sub _install_tag_handler {
-
     my $target = shift;
 
+    # we need to
+    #  1) use the target package's around() function, and
+    #  2) call it in that package's context.
+
+    # create a closure which knows about the target's around
+    # so that if namespace::clean is called on the target class
+    # we don't lose access to it.
+
+    ## no critic (ProhibitStringyEval)
+    my $around = eval( "package $target; sub { goto &around }" );
+
     install_modifier(
         $target,
         after => has => sub {
@@ -117,17 +105,8 @@ sub _install_tag_handler {
 
             $attrs = ref $attrs ? $attrs : [$attrs];
 
-            my $target = caller;
-
             my @tags = @{ $TAGSTORE{$target} };
 
-            # we need to
-            #  1) use the target package's around() function, and
-            #  2) call it in that package's context.
-
-            ## no critic (ProhibitStringyEval)
-            my $around = eval( "package $target; sub { goto &around }" );
-
             $around->(
                 "_tag_list" => sub {
                     my $orig = shift;
@@ -142,9 +121,12 @@ sub _install_tag_handler {
                 } );
 
         } );
-
 }
 
+# Moo::Role won't compose anything before it was used into a consuming
+# package. Don't want import to be consumed.
+use Moo::Role;
+
 use Sub::Name 'subname';
 
 my $can = sub { ( shift )->next::can };
@@ -157,11 +139,9 @@ my $can = sub { ( shift )->next::can };
 # but note that djerius' published solution was incomplete.
 around _tag_list => sub {
 
-    # 1. call &$orig to handle tag role compositions into the current
-    #    class
+    # 1. call &$orig to handle tag role compositions into the current class
 
-    # 2. call up the inheritance stack to handle parent class tag role
-    #    compositions.
+# 2. call up the inheritance stack to handle parent class tag role compositions.
 
     my $orig    = shift;
     my $package = caller;
@@ -178,8 +158,8 @@ use namespace::clean -except => qw( import );
 # _tags can't be lazy; we must resolve the tags and attributes at
 # object creation time in case a role is modified after this object
 # is created, as we scan both clsses and roles to gather the tags.
-# classes are immutable after the first instantiation
-# of an object, but roles aren't.
+# classes should be immutable after the first instantiation
+# of an object (but see RT#101631), but roles aren't.
 
 # We also need to identify when a role has been added to an *object*
 # which adds tagged attributes.  TODO: make this work.
@@ -195,7 +175,10 @@ sub _build_cache {
 
     my $class = shift;
 
-    # returned cached tags if available.
+    # returned cached tags if available.  Note that as of Moo v.1.006001
+    # instantiated classes may still have attributes composed into them
+    # (i.e., they're not fully immutable, see RT#101631), but that
+    # is acknowledged as a bug, not a feature, so we don't support that.
     return $TAGCACHE{$class} if $TAGCACHE{$class};
 
     my %cache;
@@ -213,54 +196,83 @@ has _tag_cache => (
     is       => 'ro',
     init_arg => undef,
     default  => sub {
-	my $class = blessed( $_[0] );
-	return $TAGCACHE{$class} ||= $class->_build_cache;
-    }
-);
+        my $class = blessed( $_[0] );
+        return $TAGCACHE{$class} ||= $class->_build_cache;
+    } );
 
 sub _tags { blessed( $_[0] ) ? $_[0]->_tag_cache : $_[0]->_build_cache }
 
 1;
 
+#
+# This file is part of MooX-TaggedAttributes
+#
+# This software is Copyright (c) 2018 by Smithsonian Astrophysical Observatory.
+#
+# This is free software, licensed under:
+#
+#   The GNU General Public License, Version 3, June 2007
+#
+
 __END__
 
+=pod
+
+=for :stopwords Diab Jerius Smithsonian Astrophysical Observatory instantiation use'ing
+
 =head1 NAME
 
 MooX::TaggedAttributes - Add a tag with an arbitrary value to a an attribute
 
+=head1 VERSION
+
+version 0.07
 
 =head1 SYNOPSIS
 
-    # Create a Role used to apply the attributes
-    package Tags;
-    use Moo::Role;
-    use MooX::TaggedAttributes -tags => [ qw( t1 t2 ) ];
+ # define a Tag Role
+ package T1;
+ use Moo::Role;
+ 
+ use MooX::TaggedAttributes -tags => [qw( t1 t2 )];
+ 1;
 
-    # Apply the role directly to a class
-    package C1;
-    use Tags;
+ # Apply a tag role directly to a class
+ package C1;
+ use Moo;
+ use T1;
+ 
+ has c1 => ( is => 'ro', t1 => 1 );
+ 1;
 
-    has c1 => ( is => 'ro', t1 => 1 );
+ # use a tag role in another Role
+ package R1;
+ 
+ use Moo::Role;
+ use T1;
+ 
+ has r1 => ( is => 'ro', t2 => 2 );
+ 1;
 
-    my $obj = C1->new;
+ # Use a tag role which consumes a tag role in a class
+ package C2;
+ use Moo;
+ use R1;
+ 
+ has c2 => ( is => 'ro', t2 => sub { } );
+ 1;
 
-    # get the value of the tag t1, applied to attribute a1
-    $obj->_tags->{t1}{a1};
-
-    # Apply the tags to a role
-    package R1;
-    use Tag1;
-
-    has r1 => ( is => 'ro', t2 => 2 );
-
-    # Use that role in a class
-    package C2;
-    use R1;
-
-    has c2 => ( is => 'ro', t2 => sub { }  );
-
-    # get the value of the tag t2, applied to attribute c2
-    C2->new->_tags->{t2}{c2};
+ # Use our tags
+ use C1;
+ use C2;
+ 
+ use 5.010;
+ 
+ # get the value of the tag t1, applied to attribute a1
+ say C1->new->_tags->{t1}{a1};
+ 
+ # get the value of the tag t2, applied to attribute c2
+ say C2->new->_tags->{t2}{c2};
 
 =head1 DESCRIPTION
 
@@ -272,20 +284,24 @@ which tags, and what the values are.
 
 To define a set of tags, create a special I<tag role>:
 
-    package T1;
-    use Moo::Role;
-    use MooX::TaggedAttributes -tags => [ 't1' ];
-
-    has a1 => ( is => 'ro', t1 => 'foo' );
+ package T1;
+ use Moo::Role;
+ use MooX::TaggedAttributes -tags => [ 't1' ];
+ 
+ has a1 => ( is => 'ro', t1 => 'foo' );
+ 
+ 1;
 
 If there's only one tag, it can be passed directly without being
 wrapped in an array:
 
-    package T2;
-    use Moo::Role;
-    use MooX::TaggedAttributes -tags => 't2';
-
-    has a2 => ( is => 'ro', t2 => 'bar' );
+ package T2;
+ use Moo::Role;
+ use MooX::TaggedAttributes -tags => 't2';
+ 
+ has a2 => ( is => 'ro', t2 => 'bar' );
+ 
+ 1;
 
 A tag role is a standard B<Moo::Role> with added machinery to track
 attribute tags.  As shown, attributes may be tagged in the tag role
@@ -300,37 +316,45 @@ existing tags, but won't provide the ability to tag new attributes.
 
 This is correct:
 
-    package R2;
-    use Moo::Role;
-    use T1;
+ package R2;
+ use Moo::Role;
+ use T1;
+ 
+ has r2 => ( is => 'ro', t1 => 'foo' );
+ 1;
 
-    has r2 => ( is => 'ro', t1 => 'foo' );
-
-    package R3;
-    use Moo::Role;
-    use R3;
-
-    has r3 => ( is => 'ro', t1 => 'foo' );
+ package R3;
+ use Moo::Role;
+ use R3;
+ 
+ has r3 => ( is => 'ro', t1 => 'foo' );
+ 1;
 
 The same goes for classes:
 
-    package C2;
-    use Moo;
-    use T1;
-
-    has c2 => ( is => 'ro', t1 => 'foo' );
+ package C1;
+ use Moo;
+ use T1;
+ 
+ has c1 => ( is => 'ro', t1 => 'foo' );
+ 1;
 
 Combining tag roles is as simple as B<use>'ing them in the new role:
 
-    package T12;
-    use T1;
-    use T2;
+ package T12;
+ 
+ use Moo::Role;
+ use T1;
+ use T2;
+ 
+ 1;
 
-    package C2;
-    use Moo;
-    use T12;
-
-    has c2 => ( is => 'ro', t1 => 'foo', t2 => 'bar' );
+ package C2;
+ use Moo;
+ use T12;
+ 
+ has c2 => ( is => 'ro', t1 => 'foo', t2 => 'bar' );
+ 1;
 
 =head2 Accessing tags
 
@@ -338,27 +362,30 @@ Classes and objects are provided a B<_tags> method which returns a
 hash of hashes keyed off of the tags and attribute names.  For
 example, for the following code:
 
-    package T;
-    use Moo::Role;
-    use MooX::TaggedAttributes -tags => [ qw( t1 t2 ) ];
+ package T;
+ use Moo::Role;
+ use MooX::TaggedAttributes -tags => [qw( t1 t2 )];
+ 1;
 
-    package C;
-    use Moo;
-    use T;
+ package C;
+ use Moo;
+ use T;
+ 
+ has a => ( is => 'ro', t1 => 2 );
+ has b => ( is => 'ro', t2 => 'foo' );
+ 1;
 
-    has a => ( is => 'ro', t1 => 2 );
-    has b => ( is => 'ro', t2 => 'foo' );
+The tag structure returned by  C<< C->_tags >>
 
-The tag structure returned by either of the following
+ { t1 => { a => 2 }, t2 => { b => "foo" } }
 
-    C->_tags
-    C->new->_tags
 
-looks like
+and C<< C->new->_tags >>
 
-    { t1 => { a => 2 },
-      t2 => { b => 'foo' },
-    }
+ { t1 => { a => 2 }, t2 => { b => "foo" } }
+
+
+are identical.
 
 =head1 BUGS AND LIMITATIONS
 
@@ -367,29 +394,32 @@ looks like
 If a role with tagged attributes is applied to an object, the
 tags for those attributes are not visible.
 
+=head1 BUGS
 
+Please report any bugs or feature requests on the bugtracker website
+L<https://rt.cpan.org/Public/Dist/Display.html?Name=MooX-TaggedAttributes>
+or by email to
+L<bug-MooX-TaggedAttributes@rt.cpan.org|mailto:bug-MooX-TaggedAttributes@rt.cpan.org>.
 
-Please report any bugs or feature requests to
-C<bug-moox-taggedattributes@rt.cpan.org>, or through the web interface at
-L<https://rt.cpan.org/Public/Dist/Display.html?Name=MooX-TaggedAttributes>.
+When submitting a bug or request, please include a test-file or a
+patch to an existing test-file that illustrates the bug or desired
+feature.
 
-=head1 LICENSE AND COPYRIGHT
+=head1 SOURCE
 
-Copyright (c) 2015 The Smithsonian Astrophysical Observatory
-
-MooX::TaggedAttributes is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or (at
-your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program.  If not, see L<http://www.gnu.org/licenses/>.
+The development version is on github at L<https://github.com/djerius/moox-taggedattributes>
+and may be cloned from L<git://github.com/djerius/moox-taggedattributes.git>
 
 =head1 AUTHOR
 
-Diab Jerius  E<lt>djerius@cpan.orgE<gt>
+Diab Jerius <djerius@cpan.org>
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is Copyright (c) 2018 by Smithsonian Astrophysical Observatory.
+
+This is free software, licensed under:
+
+  The GNU General Public License, Version 3, June 2007
+
+=cut
